#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLINELEN 512
#define MAXDESTLEN (2 * MAXLINELEN)

/* Prima versione: ritorna -1 se le stringhe da concatenare sono troppo lunghe oppure
 * ritorna un intero positivo pari alla lunghezza della stringa destinazione.
 */
int mystrcat1(const char *s1, const char *s2, char *dest);

/* questa versione e` errata: restituisce un riferimento a una variabile di tipo 
 * stringa (vettore di caratteri) definita localmente all'interno della funzione.
 * Quando la funzione termina, lo spazio riservato alla stringa viene liberato 
 * e quindi la zona di memoria potrebbe non essere piu` valida!
 */
char *mystrcat2(const char *s1, const char *s2);

/* Terza versione: restituisce un puntatore a una zona allocata dinamicamente: e` necessario
 * che il chiamante liberi lo spazio puntato dal valore di ritorno quando non piu` utilizzato.
 */
char *mystrcat3(const char *s1, const char *s2);

int main()
{
	char s1[MAXLINELEN], s2[MAXLINELEN];
	char dest[MAXDESTLEN];
	char dest2[MAXDESTLEN];
	char *dst;

	printf("Inserisci s1: ");
	fgets(s1, MAXLINELEN, stdin);

	printf("Inserisci s2: ");
	fgets(s2, MAXLINELEN, stdin);

	/* non voglio il newline alla fine di s2 e s1 */
	s1[strlen(s1) - 1] = '\0';
	s2[strlen(s2) - 1] = '\0';

	if(mystrcat1(s1, s2, dest) < 0)
		printf("Errore: lunghezza s1 e s2 eccessiva (%d + %d)\n", strlen(s1), strlen(s2));
	else
		printf("strcat1: \"%s\" + \"%s\" = \"%s\"\n", s1, s2, dest);
	
	strcpy(dest2, mystrcat2(s1, s2));

	printf("strcat2: \"%s\" + \"%s\" = \"%s\"\n", s1, s2, dest2);

	dst = mystrcat3(s1, s2);

	printf("strcat3: \"%s\" + \"%s\" = \"%s\"\n", s1, s2, dst);

	/* non dimenticare che abbiamo allocato dinamicamente lo spazio per 
	 * la stringa concatenata nella funzione mystrcat3. In questo caso
	 * e` molto importante documentare la funzione mystrcat3!
	 */
	free(dst);

	if(strlen(s1) + strlen(s2) > MAXDESTLEN - 1)
		return EXIT_SUCCESS;
	else /* non ho potuto concatenare */
		return EXIT_FAILURE;
}


int mystrcat1(const char *s1, const char *s2, char dest[])
{
	int i;
	if(strlen(s1) + strlen(s2) > MAXDESTLEN - 1) /* -1 perche' serve spazio per '\0' finale */
		return -1;
	for(i = 0; i < strlen(s1); i++)
		dest[i] = s1[i];
	for(i = 0; i <  strlen(s2); i++)
		dest[i + strlen(s1)] = s2[i];
	dest[i + strlen(s1)] = '\0';	
	return i;
}

char *mystrcat2(const char *s1, const char *s2)
{
	int i;
	int s1len = strlen(s1);
	int s2len = strlen(s2);
	char dest[MAXDESTLEN] = "";
	if(s1len + s2len < MAXDESTLEN - 1)
	{
		for(i = 0; i < s1len; i++)
			dest[i] = s1[i];
		for(i = 0; i < s2len; i++)
			dest[i + s1len] = s2[i];
		dest[i + s1len] = '\0';	
	}
	return dest;
}

char *mystrcat3(const char *s1, const char *s2)
{
	int i;
	int s1len = strlen(s1);
	/* alloco lo spazio dinamicamente. Devo ricordarmi di riservare lo spazio in piu` per il carattere 
	 * di terminazione.
	 */
	char *dest = (char *) malloc(sizeof(char) * s1len + sizeof(char) * strlen(s2) + 1 * sizeof(char)); 

	for(i = 0; i < s1len; i++)
		dest[i] = s1[i];
	for(i = 0; i < strlen(s2); i++)
		dest[i + s1len] = s2[i];
	dest[i + s1len] = '\0';
	return dest;
}



